@@ -9,18 +9,8 @@ gp_interleaver;
 ldpc;
 
 #{
-  TODO: 
-    [ ] compute SNR and PAPR
-    [ ] SSB bandpass filtering
-    [ ] way to simulate aquisition and demod
-    [ ] testframe based, maybe repeat every 10 seconds
-        + work out which pattern we match to sync up
-    [ ] acquisition curves
-        + plot error versus freq and timing offset
-        + plot pro acquisition versus freq offset, timing and freq sep and together
-        + plot total acquist prob at various SNRs ... maybe mean number of frames to sync?
-    [ ] replace genie EsNo est used for ldpc dec
-    [ ] clean up text 
+  TODO:
+    [ ] run_sim neeeds to be refactored for coded operation at Nc=17 with UW
 #}
 
 function [sim_out rx states] = run_sim(sim_in)
@@ -112,14 +102,15 @@ function [sim_out rx states] = run_sim(sim_in)
     HRA = sim_in.ldpc_code;
     [aNr aNc] = size(HRA);
     rate = states.rate = (aNc-aNr)/aNc;
-    assert(aNc == Nbitsperframe, "Dude: Num cols of LDPC HRA must == Nbitsperframe");
+    Ndatabitsperframe = Nbitsperframe;
+    assert(aNc == Ndatabitsperframe, "Dude: Num cols of LDPC HRA must == Nbitsperframe");
     [H_rows, H_cols] = Mat2Hrows(HRA); 
     code_param.H_rows = H_rows; 
     code_param.H_cols = H_cols;
     code_param.P_matrix = [];
     code_param.data_bits_per_frame = length(code_param.H_cols) - length( code_param.P_matrix ); 
     code_param.code_bits_per_frame = aNc;
-    assert(aNr == Nbitsperframe*rate);
+    assert(aNr == Ndatabitsperframe*rate);
 
     modulation = states.ldpc_modulation = 'QPSK';
     mapping = states.ldpc_mapping = 'gray';
@@ -182,8 +173,12 @@ function [sim_out rx states] = run_sim(sim_in)
     % generate tx bits, optionaly LDPC encode, and modulate as QPSK symbols
     % note for reasons unknown LdpcEncode() returns garbage if we use > 0.5 rather than round()
 
-    tx_data_bits = round(rand(1,Nbits*rate));
+    %tx_data_bits = round(rand(1,Nbits*rate));
 
+    % std test frame so we can x-check
+    
+    tx_data_bits = create_ldpc_test_frame;
+    
     tx_bits = []; tx_symbols = [];
     for f=1:Nframes
       st = (f-1)*Nbitsperframe*rate+1; en = st + Nbitsperframe*rate - 1;
@@ -199,7 +194,10 @@ function [sim_out rx states] = run_sim(sim_in)
         assert(length(codeword) == Nbitsperframe);
       else
         % uncoded mode
-        codeword = tx_data_bits(st:en);
+        codeword = tx_data_bits;
+        if isfield(sim_in, "uw_debug")
+          codeword(states.uw_ind) = states.tx_uw;
+        end
       end
       tx_bits = [tx_bits codeword];
       for b=1:2:Nbitsperframe
@@ -269,10 +267,23 @@ function [sim_out rx states] = run_sim(sim_in)
     phase_offset = woffset*(1:Nsam) + 0.5*dwoffset*((1:Nsam).^2);
     rx = rx .* exp(j*phase_offset);
    
+    if isfield(sim_in, "initial_noise_sams")
+      rx = [zeros(1, sim_in.initial_noise_sams) rx];
+      Nsam = length(rx);
+    end
+    
     noise = sqrt(variance)*(0.5*randn(1,Nsam) + j*0.5*randn(1,Nsam));
     snrdB = 10*log10(var(rx)/var(noise)) + 10*log10(8000) - 10*log10(3000);
     rx += noise;
 
+    % interfering carrier
+
+    % rx += 0.04*cos((1:length(rx))*states.w(10));
+    
+    % gain
+    
+    rx *= sim_in.gain;
+    
     % some spare samples at end to avoid overflow as est windows may poke into the future a bit
 
     rx = [rx zeros(1,Nsamperframe)];
@@ -285,7 +296,9 @@ function [sim_out rx states] = run_sim(sim_in)
     foff_est_hz_log = [];
     Nerrs_log = []; Nerrs_coded_log = [];
     rx_bits = []; rx_np = []; rx_amp = [];
-
+    sig_var_log = []; noise_var_log = [];
+    uw_errors_log = [];
+    
     % reset some states for each EbNo simulation point
 
     states.sample_point = states.timing_est = 1;
@@ -309,21 +322,34 @@ function [sim_out rx states] = run_sim(sim_in)
 
       lnew = min(Nsam-prx,states.nin);
       rxbuf_in = zeros(1,states.nin);
-
+      
       if lnew
         rxbuf_in(1:lnew) = rx(prx:prx+lnew-1);
       end
       prx += states.nin;
 
       [arx_bits states aphase_est_pilot_log arx_np arx_amp] = ofdm_demod(states, rxbuf_in);
-
+     
       rx_bits = [rx_bits arx_bits]; rx_np = [rx_np arx_np]; rx_amp = [rx_amp arx_amp];
+
+      % note: only supported in ldpc_en = 0 Nc=17 atm, see debug_false_sync()
+      
+      if isfield(sim_in, "uw_debug") 
+        rx_uw = arx_bits(states.uw_ind);
+        uw_errors = sum(xor(states.tx_uw,rx_uw));
+        if verbose
+          printf("f: %d uw_errors: %d\n", f, uw_errors);
+        end
+        uw_errors_log = [uw_errors_log uw_errors];
+      end
+      
       timing_est_log = [timing_est_log states.timing_est];
       delta_t_log = [delta_t_log states.delta_t];
       foff_est_hz_log = [foff_est_hz_log states.foff_est_hz];
       phase_est_pilot_log = [phase_est_pilot_log; aphase_est_pilot_log];
+      sig_var_log = [sig_var_log states.sig_var];
+      noise_var_log = [noise_var_log states.noise_var];
     end
-
     assert(length(rx_bits) == Nbits);
 
     % Optional de-interleave on rx QPSK symbols
@@ -363,6 +389,7 @@ function [sim_out rx states] = run_sim(sim_in)
     % Per-modem frame error log and optional LDPC/diversity error stats
 
     Terrs_coded = 0; Tpackets_coded = 0; Tpacketerrs_coded = 0; sim_out.error_positions = [];
+    
     for f=1:Nframes
       st = (f-1)*Nbitsperframe+1; en = st + Nbitsperframe - 1;
       Nerrs_log(f) = sum(xor(tx_bits(st:en), rx_bits(st:en)));
@@ -370,13 +397,15 @@ function [sim_out rx states] = run_sim(sim_in)
       st = (f-1)*Nbitsperframe/bps + 1;
       en = st + Nbitsperframe/bps - 1;
       r = rx_np(st:en); fade = rx_amp(st:en);
-
+      
       % optional LDPC decode
      
       if ldpc_en
-        % note we put ceiling on EsNo as decoder misbehaives with high EsNo
 
-        rx_codeword = ldpc_dec(code_param, max_iterations, demod_type, decoder_type, r, min(EsNo,30), fade);
+        % scale based on amplitude ests
+
+        mean_amp = states.mean_amp;
+        rx_codeword = ldpc_dec(code_param, max_iterations, demod_type, decoder_type, r/mean_amp, min(EsNo,30), fade/mean_amp);
       end
 
       % optional diversity demod
@@ -424,26 +453,34 @@ function [sim_out rx states] = run_sim(sim_in)
 
     % print results of this simulation point to the console
 
-    if ldpc_en || diversity_en
-      printf("Coded EbNodB: % -4.1f BER: %5.4f Tbits: %5d Terrs: %5d PER: %5.4f Tpackets: %5d Tpacket_errs: %5d\n", 
-              EbNodB(nn), Terrs_coded/(Nbits*rate), Nbits*rate, Terrs_coded, 
-              Tpacketerrs_coded/Tpackets_coded, Tpackets_coded, Tpacketerrs_coded);
-    end
-    EbNodB_raw = EbNodB(nn) + 10*log10(rate);
-    printf("Raw EbNodB..: % -4.1f BER: %5.4f Tbits: %5d Terrs: %5d PER: %5.4f Tpackets: %5d Tpacket_errs: %5d\n", 
-           EbNodB_raw, Terrs/Nbits, Nbits, Terrs,
-           Tpacketerrs/Tpackets, Tpackets, Tpacketerrs);
+    if verbose
+      if ldpc_en || diversity_en
+        printf("Coded EbNodB: % -4.1f BER: %5.4f Tbits: %5d Terrs: %5d PER: %5.4f Tpackets: %5d Tpacket_errs: %5d\n", 
+                EbNodB(nn), Terrs_coded/(Nbits*rate), Nbits*rate, Terrs_coded, 
+                Tpacketerrs_coded/Tpackets_coded, Tpackets_coded, Tpacketerrs_coded);
+      end
+      EbNodB_raw = EbNodB(nn) + 10*log10(rate);
+      printf("Raw EbNodB..: % -4.1f BER: %5.4f Tbits: %5d Terrs: %5d PER: %5.4f Tpackets: %5d Tpacket_errs: %5d\n", 
+             EbNodB_raw, Terrs/Nbits, Nbits, Terrs,
+             Tpacketerrs/Tpackets, Tpackets, Tpacketerrs);
+      EsNo = mean(sig_var_log)/mean(noise_var_log);
+      %printf("Es/No est dB: % -4.1f\n", 10*log10(EsNo));
+      sim_out.snrdB(nn) = snrdB;
+      sim_out.snr_estdB(nn) = 10*log10(EsNo) + 10*log10(Nc*Rs/3000);
+      
+      % returns results for plotting curves
 
-    % returns results for plotting curves
-
-    if ldpc_en || diversity_en
-      sim_out.ber(nn) = Terrs_coded/(Nbits*rate); 
-      sim_out.per(nn) = Tpacketerrs_coded/Tpackets_coded; 
-    else
-      sim_out.ber(nn) = Terrs/Nbits; 
-      sim_out.per(nn) = Tpacketerrs/Tpackets; 
+      if ldpc_en || diversity_en
+        sim_out.ber(nn) = Terrs_coded/(Nbits*rate); 
+        sim_out.per(nn) = Tpacketerrs_coded/Tpackets_coded; 
+      else
+        sim_out.ber(nn) = Terrs/Nbits; 
+        sim_out.per(nn) = Tpacketerrs/Tpackets; 
+      end
     end
 
+    sim_out.uw_errors_log = uw_errors_log;
+    
     % Optional plots, mostly used with run-single
 
     if verbose
@@ -479,20 +516,31 @@ function [sim_out rx states] = run_sim(sim_in)
         subplot(212)
         stem(Nerrs_coded_log/(Nbitsperframe*rate));
         title("Coded BER/frame");
-     else
+      else
         title("BER/frame");
         stem(Nerrs_log/Nbitsperframe);
       end
 
       figure(6)
-      Tx = abs(fft(tx(1:Nsam).*hanning(Nsam)'));
+      Tx = abs(fft(rx(1:Nsam).*hanning(Nsam)'));
       Tx_dB = 20*log10(Tx);
       dF = Fs/Nsam;
       plot((1:Nsam)*dF, Tx_dB);
       mx = max(Tx_dB);
       axis([0 Fs/2 mx-60 mx])
 
-     
+      figure(7); clf;
+      plot(10*log10(sig_var_log),'b;Es;');
+      hold on;
+      plot(10*log10(noise_var_log),'r;No;');
+      snr_estdB = 10*log10(sig_var_log) - 10*log10(noise_var_log) + 10*log10(Nc*Rs/3000);
+      snr_est_smoothed_dB = filter(0.1,[1 -0.9],snr_estdB);
+      plot(snr_estdB,'g;SNR3k;');
+      plot(snr_est_smoothed_dB,'c;SNR3k smooth;');
+      
+      hold off;
+      title('Signal and Noise Power estimates');
+      
 #{
       if hf_en
         figure(4); clf; 
@@ -511,8 +559,6 @@ function [sim_out rx states] = run_sim(sim_in)
         plot(angle(hf_model(:,2:Nc+1)));
       end
 #}
-
-
     end
 
   end
@@ -525,17 +571,18 @@ function run_single(EbNodB = 100, error_pattern_filename);
   sim_in.Rs = 1/Ts; sim_in.bps = 2; sim_in.Nc = 16; sim_in.Ns = 8;
 
   sim_in.Nsec = (sim_in.Ns+1)/sim_in.Rs;  % one frame, make sure sim_in.interleave_frames = 1
-  %sim_in.Nsec = 1;
+  sim_in.Nsec = 10;
 
-  sim_in.EbNodB = EbNodB;
-  sim_in.verbose = 2;
+  sim_in.EbNodB = 40;
+  sim_in.verbose = 1;
   sim_in.hf_en = 0;
   sim_in.foff_hz = 0;
   sim_in.dfoff_hz_per_sec = 0.00;
   sim_in.sample_clock_offset_ppm = 0;
-
-  sim_in.timing_en = 1;
-  sim_in.foff_est_en = 1;
+  sim_in.gain = 1;
+  
+  sim_in.timing_en = 0;
+  sim_in.foff_est_en = 0;
   sim_in.phase_est_en = 1;
 
   load HRA_112_112.txt
@@ -543,7 +590,6 @@ function run_single(EbNodB = 100, error_pattern_filename);
   sim_in.ldpc_en = 1;
 
   sim_in.interleave_frames = 1;
-
   %sim_in.diversity_en = 1;
 
   sim_out = run_sim(sim_in);
@@ -578,6 +624,8 @@ function run_curves
 
   sim_in.verbose = 0;
   sim_in.foff_hz = 0;
+  sim_in.gain = 1;
+  
   sim_in.timing_en = 1;
   sim_in.foff_est_en = 1;
   sim_in.phase_est_en = 1;
@@ -841,143 +889,430 @@ function run_curves_estimators
 end
 
 
-% Run an acquisition test, returning vectors of estimation errors
+% Plot SNR actual and estimated for various channels
+%   Note no acquistion, as this can upset results, e.g. if
+%   sync is lost.  We average SNR over entire run, in practice
+%   there will be some sort of IIR averager.
 
-function [delta_ct delta_foff] = acquisition_test(Ntests=10, EbNodB=100, foff_hz=0, hf_en=0, fine_en=0)
+function run_curves_snr
+
+  Nsec_awgn = 30;
+  Nsec_hf = 60;
+
+  % waveform
+
+  Ts = 0.018; sim_in.Tcp = 0.002; 
+  sim_in.Rs = 1/Ts; sim_in.bps = 2; sim_in.Nc = 16; sim_in.Ns = 8;
+
+  % simulation parameters
+
+  sim_in.verbose = 1; sim_in.foff_hz = 0; sim_in.ldpc_en = 0; sim_in.gain = 1;
+  sim_in.phase_est_en = sim_in.timing_en = sim_in.foff_est_en = 1;
+
+  % AWGN simulation
+
+  sim_in.EbNodB = 0:3:12;
+  sim_in.hf_en = 0; sim_in.Nsec = Nsec_awgn;
+
+  awgn = run_sim(sim_in);
+  
+  % HF simulations
+
+  sim_in.hf_en = 1; sim_in.Nsec = Nsec_hf; sim_in.dopplerSpreadHz = 1; hf_fast = run_sim(sim_in);
+  sim_in.dopplerSpreadHz = 0.5; hf_mid = run_sim(sim_in);
+  sim_in.dopplerSpreadHz = 0.2; hf_slow = run_sim(sim_in);
+  
+  figure(1); clf;
+  plot(sim_in.EbNodB, awgn.snrdB,'b+-;AWGN SNR;');
+  hold on;
+  plot(sim_in.EbNodB, awgn.snr_estdB   ,'g+-;AWGN SNR est;');
+  plot(sim_in.EbNodB, hf_slow.snr_estdB,'k+-;HF 0.2 Hz SNR est;');
+  plot(sim_in.EbNodB, hf_mid.snr_estdB ,'r+-;HF 0.5 Hz SNR est;');
+  plot(sim_in.EbNodB, hf_fast.snr_estdB,'c+-;HF 1.0 Hz SNR est;');
+  hold off;
+  xlabel('Eb/No (dB)');
+  ylabel('SNR (dB)');
+  grid;
+  legend('boxoff');
+  title('SNR Actual and Estimated');
+  legend("location", "northwest");
+  print('-deps', '-color', "ofdm_dev_snr.eps")
+
+end
+
+
+% Run an acquisition test, returning vectors of estimation errors.
+% Generates a vector of noise followed by continous rx signal to
+% simulate signal starting.  Freq est will improve over time as we
+% have an averaging statistic.  We then measure how long it takes to
+% get good timing and freq estimates.
+
+function [delta_ct delta_foff timing_mx_log] = acquisition_test(Ntests=10, EbNodB=100, foff_hz=0, hf_en=0, verbose=0)
 
   Ts = 0.018; 
   sim_in.Tcp = 0.002; 
-  sim_in.Rs = 1/Ts; sim_in.bps = 2; sim_in.Nc = 16; sim_in.Ns = 8;
+  sim_in.Rs = 1/Ts; sim_in.bps = 2; sim_in.Nc = 17; sim_in.Ns = 8;
 
-  sim_in.Nsec = Ntests*(sim_in.Ns+1)/sim_in.Rs;
+  sim_in.Nsec = (Ntests+1)*(sim_in.Ns+1)/sim_in.Rs;
 
-  sim_in.EbNodB = EbNodB;
-  sim_in.verbose = 2;
+  #{
+    Notes:
+      1) uncoded modem operating point, e.g -1dB for AWGN
+      2) run_sim adds complex noise, when we take the real() below, this relects
+         the -ve noise over to the +ve side, increasing the noise by 3dB.  In
+         ofdm_tx.m and friends, we add real nosie that is correctly scaled so
+         no problemo.  This means we need to increase the Eb/No below by 3dB
+         to ensure the correct level of nosie ta the input of the timing_est.         
+  #}
+  
+  sim_in.EbNodB = EbNodB + 3;
+  sim_in.verbose = 0;
   sim_in.hf_en = hf_en;
-  sim_in.foff_hz = foff_hz; 
-  sim_in.timing_en = 0;
-  sim_in.foff_est_en = 0;
-  sim_in.phase_est_en = 0;
+  sim_in.foff_hz = foff_hz;
+  sim_in.gain = 1;
+  sim_in.timing_en = 1;
+  sim_in.foff_est_en = 1;
+  sim_in.phase_est_en = 1;
+  sim_in.ldpc_en = 0;
 
+  % optionally stick a bunch of noise in front of signal to confuse things
+  
+  sim_in.initial_noise_sams = 0;
+  
   [sim_out rx states] = run_sim(sim_in);
 
+  states.verbose = verbose;
+  
   % set up acquistion 
 
-  Nsamperframe = states.Nsamperframe;
+  Nsamperframe = states.Nsamperframe; M = states.M; Ncp = states.Ncp;
   rate_fs_pilot_samples = states.rate_fs_pilot_samples;
 
   % test fine or acquisition over test signal
-  #{
-    fine: - start with coarse timing instant
-          - on each frame est timing a few samples about that point
-          - update timing instant
 
-    corr: - where is best plcase to sample
-          - just before end of symbol?
-          - how long should sequence be?
-          - add extra?
-          - aim for last possible moment?
-          - man I hope IL isn't too big.....
-  #}
+  delta_ct = []; delta_foff = []; timing_mx_log = []; foff_metric_log = [];
 
-  delta_ct = []; delta_foff = [];
-
-  if fine_en
-
-    window_width = 5;                   % search +/-2 samples from current timing instant
-    timing_instant = Nsamperframe+1;    % start at correct instant for AWGN
-                                        % start at second frame so we can search -2 ... +2
-
-    while timing_instant < (length(rx) - (Nsamperframe + length(rate_fs_pilot_samples) + window_width))
-      st = timing_instant - ceil(window_width/2); 
-      en = st + Nsamperframe-1 + length(rate_fs_pilot_samples) + window_width-1;
-      [ft_est foff_est] = coarse_sync(states, rx(st:en), rate_fs_pilot_samples);
-      printf("ft_est: %d timing_instant %d %d\n", ft_est, timing_instant, mod(timing_instant, Nsamperframe));
-      timing_instant += Nsamperframe + ft_est - ceil(window_width/2);
-      delta_t = [delta_ft ft_est - ceil(window_width/2)];
-    end
-  else
-    % for coarse simulation we just use contant window shifts
+  
+    % coarse acquiistion test.  We have no idea of timing or freq
+    % offset for coarse we just use constant window shifts to simulate
+    % a bunch of trials, this allows averaging of freq est
+    % metric over time as we receive more and more frames
 
     st = 0.5*Nsamperframe; 
-    en = 2.5*Nsamperframe - 1;
-    ct_target = Nsamperframe/2;
+    en = 2.5*Nsamperframe - 1;    % note this gives Nsamperframe possibilities for coarse timing
 
-    for w=1:Nsamperframe:length(rx)-3*Nsamperframe
-      %st = w+0.5*Nsamperframe; en = st+2*Nsamperframe-1;
-      %[ct_est foff_est] = coarse_sync(states, rx(st:en), rate_fs_pilot_samples);
-      [ct_est foff_est] = coarse_sync(states, rx(w+st:w+en), rate_fs_pilot_samples);
+    % actual known position of correct coarse timing
+
+    ct_target = mod(sim_in.initial_noise_sams + Nsamperframe/2, Nsamperframe);
+
+    i = 1;
+    states.foff_metric = 0;
+    for w=1:Nsamperframe:length(rx)-4*Nsamperframe
+      [ct_est timing_valid timing_mx] = est_timing(states, real(rx(w+st:w+en)), rate_fs_pilot_samples);
+      [foff_est states] = est_freq_offset(states, real(rx(w+st:w+en)), rate_fs_pilot_samples, ct_est);
       if states.verbose
-        printf("ct_est: %4d foff_est: %3.1f\n", ct_est, foff_est);
+        printf("i: %2d w: %5d ct_est: %4d foff_est: %5.1f timing_mx: %3.2f timing_vld: %d\n", i++, w, ct_est, foff_est, timing_mx, timing_valid);
       end
 
       % valid coarse timing ests are modulo Nsamperframe
 
       delta_ct = [delta_ct ct_est-ct_target];
       delta_foff = [delta_foff (foff_est-foff_hz)];
+      timing_mx_log = [timing_mx_log; timing_mx];
+      foff_metric_log = [foff_metric_log states.foff_metric];
     end
-  end
 
+  if states.verbose > 1
+    %printf("mean: %f std: %f\n", mean(delta_foff), std(delta_foff));
+    figure(1); clf; plot(timing_mx_log,'+-');
+    figure(2); clf; plot(delta_ct,'+-');
+    figure(3); clf; plot(delta_foff,'+-');
+    figure(4); clf; plot(foff_metric_log,'+');
+    figure(5); clf; plot(real(rx))
+  end
+  
 endfunction
 
 
-% Run some tests for various acquisition conditions. Probability of
-% acquistion is what matters, e.g. if it's 50% we can expect sync
-% within 2 frames
-%                 P(t)/P(f)  P(t)/P(f)
-%          Eb/No  AWGN       HF
-% +/- 25Hz -1/3   1.0/0.3    0.96/0.3 
-% +/-  5Hz -1/3   1.0/0.347  0.96/0.55 
-% +/- 25Hz  10/10 1.00/0.92  0.99/0.77
+#{
 
-function acquisition_histograms(fine_en = 0)
+   Generates aquisistion statistics for AWGN and HF channels for
+   continuous signals. Probability of acquistion is what matters,
+   e.g. if it's 50% we can expect sync within 2 frames.
+
+#}
+
+function res = acquisition_histograms(fine_en = 0, foff, EbNoAWGN=-1, EbNoHF=3, verbose=1)
   Fs = 8000;
   Ntests = 100;
 
   % allowable tolerance for acquistion
 
-  ftol_hz = 2.0;            % fine freq can track this out
+  ftol_hz = 1.5;            % we can sync up on this
   ttol_samples = 0.002*Fs;  % 2ms, ie CP length
 
-  % AWGN channel operating point
+  % AWGN channel at uncoded Eb/No operating point
 
-  [dct dfoff] = acquisition_test(Ntests, -1, -20, 0, fine_en);
+  [dct dfoff] = acquisition_test(Ntests, EbNoAWGN, foff, 0, fine_en);
 
   % Probability of acquistion is what matters, e.g. if it's 50% we can
   % expect sync within 2 frames
 
-  printf("AWGN P(time offset acq) = %3.2f\n", length(find (abs(dct) < ttol_samples))/length(dct));
+  PtAWGN = length(find (abs(dct) < ttol_samples))/length(dct);
+  printf("AWGN P(time offset acq) = %3.2f\n", PtAWGN);
   if fine_en == 0
-    printf("AWGN P(freq offset acq) = %3.2f\n", length(find (abs(dfoff) < ftol_hz))/length(dfoff));
+    PfAWGN = length(find (abs(dfoff) < ftol_hz))/length(dfoff);
+    printf("AWGN P(freq offset acq) = %3.2f\n", PfAWGN);
   end
 
-  figure(1)
-  hist(dct(find (abs(dct) < ttol_samples)))
-  title('Coarse Timing error AWGN')
+  if verbose
+    figure(1); clf;
+    hist(dct(find (abs(dct) < ttol_samples)))
+    t = sprintf("Coarse Timing Error AWGN EbNo = %3.2f foff = %3.1f", EbNoAWGN, foff);
+    title(t)
+    if fine_en == 0
+      figure(2)
+      hist(dfoff(find(abs(dfoff) < 2*ftol_hz)))
+      t = sprintf("Coarse Freq Error AWGN EbNo = %3.2f foff = %3.1f", EbNoAWGN, foff);
+      title(t);
+    end
+ end
+
+  % HF channel at uncoded operating point
+
+  [dct dfoff] = acquisition_test(Ntests, EbNoHF, foff, 1, fine_en);
+
+  PtHF = length(find (abs(dct) < ttol_samples))/length(dct);
+  printf("HF P(time offset acq) = %3.2f\n", PtHF);
   if fine_en == 0
-    figure(2)
-    hist(dfoff)
-  title('Coarse Freq error AWGN')
+    PfHF = length(find (abs(dfoff) < ftol_hz))/length(dfoff)
+    printf("HF P(freq offset acq) = %3.2f\n", PfHF);
   end
 
-  % HF channel operating point
+  if verbose
+    figure(3); clf;
+    hist(dct(find (abs(dct) < ttol_samples)))
+    t = sprintf("Coarse Timing Error HF EbNo = %3.2f foff = %3.1f", EbNoHF, foff);
+    title(t)
+    if fine_en == 0
+      figure(4)
+      hist(dfoff(find(abs(dfoff) < 2*ftol_hz)))
+      t = sprintf("Coarse Freq Error HF EbNo = %3.2f foff = %3.1f", EbNoHF, foff);
+      title(t);
+    end
+  end
+  
+  res = [PtAWGN PfAWGN PtHF PfHF];
+endfunction
 
-  [dct dfoff] = acquisition_test(Ntests, 3, -20, 1, fine_en);
 
-  printf("HF P(time offset acq) = %3.2f\n", length(find (abs(dct) < ttol_samples))/length(dct));
-  if fine_en == 0
-    printf("HF P(freq offset acq) = %3.2f\n", length(find (abs(dfoff) < ftol_hz))/length(dfoff));
+% plot some curves of Acquisition probability against EbNo and freq offset
+
+function acquistion_curves
+
+  EbNo = [-1 2 5 8];
+  %foff = [-20 -15 -10 -5 0 5 10 15 20];
+  foff = [-15 -5 0 5 15];
+  cc = ['b' 'g' 'k' 'c' 'm'];
+  
+  figure(1); clf; hold on; title('P(timing) AWGN'); xlabel('Eb/No dB'); legend('location', 'southeast');
+  figure(2); clf; hold on; title('P(freq) AWGN'); xlabel('Eb/No dB'); legend('location', 'southeast');
+  figure(3); clf; hold on; title('P(timing) HF'); xlabel('Eb/No dB'); legend('location', 'southeast');
+  figure(4); clf; hold on; title('P(freq) HF'); xlabel('Eb/No dB'); legend('location', 'southeast');
+  
+  for f = 1:length(foff)
+    afoff = foff(f);
+    res_log = [];
+    for e = 1:length(EbNo)
+      aEbNo = EbNo(e);
+      res = zeros(1,4);
+      res = acquisition_histograms(fine_en = 0, afoff, aEbNo, aEbNo+4, verbose = 0);
+      res_log = [res_log; res];
+    end
+    figure(1); l = sprintf('%c+-;%3.1f Hz;', cc(f), afoff); plot(EbNo, res_log(:,1), l);
+    figure(2); l = sprintf('%c+-;%3.1f Hz;', cc(f), afoff); plot(EbNo, res_log(:,3), l);
+    figure(3); l = sprintf('%c+-;%3.1f Hz;', cc(f), afoff); plot(EbNo+4, res_log(:,2), l);
+    figure(4); l = sprintf('%c+-;%3.1f Hz;', cc(f), afoff); plot(EbNo+4, res_log(:,4), l);
   end
 
-  figure(3)
-  hist(dct(find (abs(dct) < ttol_samples)))
-  title('Coarse Timing error HF')
-  if fine_en == 0
-    figure(4)
-    hist(dfoff)
-    title('Coarse Freq error HF')
+  figure(1); print('-deps', '-color', "ofdm_dev_acq_curves_time_awgn.eps")
+  figure(2); print('-deps', '-color', "ofdm_dev_acq_curves_freq_awgn.eps")
+  figure(3); print('-deps', '-color', "ofdm_dev_acq_curves_time_hf.eps")
+  figure(4); print('-deps', '-color', "ofdm_dev_acq_curves_freq_hf.eps")
+endfunction
+
+
+% Used to develop sync state machine - in particular a metric to show
+% we are out of sync, or have sync with a bad freq offset est, or have
+% lost modem signal
+
+function sync_metrics(x_axis = 'EbNo')
+  Fs      = 8000;
+  Ntests  = 4;
+  f_offHz = [-25:25];
+  EbNodB  = [-10 0 3 6 10 20];
+  %f_offHz = [-5:5:5];
+  %EbNodB  = [-10 0 10];
+  cc = ['b' 'g' 'k' 'c' 'm' 'b'];
+  pt = ['+' '+' '+' '+' '+' 'o'];
+  
+  mean_mx1_log = mean_dfoff_log = [];
+  for f = 1:length(f_offHz)
+    af_offHz = f_offHz(f);
+    mean_mx1_row = mean_dfoff_row = [];
+    for e = 1:length(EbNodB)
+      aEbNodB = EbNodB(e);
+      [dct dfoff timing_mx_log] = acquisition_test(Ntests, aEbNodB, af_offHz);
+      mean_mx1 = mean(timing_mx_log(:,1));
+      printf("f_offHz: %5.2f EbNodB: % 6.2f mx1: %3.2f\n", af_offHz, aEbNodB, mean_mx1);
+      mean_mx1_row = [mean_mx1_row mean_mx1];
+      mean_dfoff_row = [mean_dfoff_row mean(dfoff)];
+    end
+    mean_mx1_log = [mean_mx1_log; mean_mx1_row];
+    mean_dfoff_log = [mean_dfoff_log; mean_dfoff_row];
   end
 
+  figure(1); clf; hold on; grid;
+  if strcmp(x_axis,'EbNo')
+    for f = 1:length(f_offHz)
+      if f == 2, hold on, end;
+      leg1 = sprintf("b+-;mx1 %4.1f Hz;", f_offHz(f));
+      plot(EbNodB, mean_mx1_log(f,:), leg1)
+    end
+    hold off;
+    xlabel('Eb/No (dB)');
+    ylabel('Coefficient')
+    title('Pilot Correlation Metric against Eb/No for different Freq Offsets');
+    legend("location", "northwest"); legend("boxoff");
+    axis([min(EbNodB) max(EbNodB) 0 1.2])
+    print('-deps', '-color', "ofdm_dev_pilot_correlation_ebno.eps")
+  end
+
+  if strcmp(x_axis,'freq')
+    % x axis is freq
+
+    for e = length(EbNodB):-1:1
+      leg1 = sprintf("%c%c-;mx1 %3.0f dB;", cc(e), pt(e), EbNodB(e));
+      plot(f_offHz, mean_mx1_log(:,e), leg1)
+    end
+    hold off;
+    xlabel('freq offset (Hz)');
+    ylabel('Coefficient')
+    title('Pilot Correlation Metric against Freq Offset for different Eb/No dB');
+    legend("location", "northwest"); legend("boxoff");
+    axis([min(f_offHz) max(f_offHz) 0 1])
+    print('-deps', '-color', "ofdm_dev_pilot_correlation_freq.eps")
+
+    mean_dfoff_log
+ 
+    figure(2); clf;
+    for e = 1:length(EbNodB)
+      if e == 2, hold on, end;
+      leg1 = sprintf("+-;mx1 %3.0f dB;", EbNodB(e));
+      plot(f_offHz, mean_dfoff_log(:,e), leg1)
+    end
+    hold off;
+    xlabel('freq offset (Hz)');
+    ylabel('Mean Freq Est Error')
+    title('Freq Est Error against Freq Offset for different Eb/No dB');
+    axis([min(f_offHz) max(f_offHz) -5 5])
+  end
+  
+endfunction
+
+
+% during development it was discovered demod could obtain a flase sync with no UW
+% errors at +/- 7Hz, approx the frame rate.  This function is used to explore that
+
+function debug_false_sync(EbNodB = 100)
+  Ts = 0.018; 
+  sim_in.Tcp = 0.002; 
+  sim_in.Rs = 1/Ts; sim_in.bps = 2; sim_in.Nc = 17; sim_in.Ns = 8;
+
+  sim_in.Nsec = (sim_in.Ns+1)/sim_in.Rs;  % one frame, make sure sim_in.interleave_frames = 1
+  sim_in.Nsec = 1;
+
+  sim_in.EbNodB = 40;
+  sim_in.verbose = 0;
+  sim_in.hf_en = 0;
+  sim_in.foff_hz = 0;
+  sim_in.dfoff_hz_per_sec = 0.00;
+  sim_in.sample_clock_offset_ppm = 0;
+  sim_in.gain = 1;
+  
+  sim_in.timing_en = 0;
+  sim_in.foff_est_en = 0;
+  sim_in.phase_est_en = 1;
+
+  load HRA_112_112.txt
+  sim_in.ldpc_code = HRA_112_112;
+  sim_in.ldpc_en = 0;
+
+  %sim_in.interleave_frames = 1;
+  %sim_in.diversity_en = 1;
+
+  sim_in.uw_debug = 1;
+
+  foff = -25:0.5:25;
+  for f=1:length(foff)
+    sim_in.foff_hz = foff(f);
+    sim_out = run_sim(sim_in);
+    min_uw_errors(f) = min(sim_out.uw_errors_log(2:end-1));
+    printf("f: %4.2f %2d\n", foff(f), min_uw_errors(f));
+  end
+  figure(1); clf;
+  plot(foff, min_uw_errors);
+  title('UW errors versus freq offset');
+  xlabel('Freq Offset (Hz)');
+end
+
+
+% Using an input raw file, plot frame by frame metric information,
+% used to debug false syncs
+
+function metric_fbf(fn, Nsec)
+  Ts = 0.018; 
+  states = ofdm_init(bps=2, Rs=1/Ts, Tcp=0.002, Ns=8, Nc=17);
+  ofdm_load_const;
+  states.verbose = 2;
+
+  % factor of 2 as input is a real valued signal
+
+  Ascale = states.amp_scale/2; 
+  f = fopen(fn,"rb"); rx = fread(f,Inf,"short")'/Ascale; fclose(f);
+  if (nargin == 2) && (length(rx) > Nsec*Fs)
+    rx = rx(1:Nsec*Fs);
+  end
+  Nsam = length(rx);
+  %bpf_coeff = make_ofdm_bpf(write_c_header_file=0);
+  %rx = filter(bpf_coeff,1,rx);
+  
+  st = 0.5*Nsamperframe; 
+  en = 2.5*Nsamperframe - 1;    % note this gives Nsamperframe possibilities for coarse timing
+
+  i = 1; w_log = timing_mx_log = av_level_log = [];
+  states.foff_metric = 0;
+  for w=1:Nsamperframe:length(rx)-4*Nsamperframe
+    printf("%3d %5d", i,w);
+    if i == 30
+      states.verbose = 3;
+    else
+      states.verbose = 2;
+    end  
+    [ct_est timing_valid timing_mx av_level] = est_timing(states, real(rx(w+st:w+en)), states.rate_fs_pilot_samples);
+    i++;
+    w_log = [w_log w];
+    timing_mx_log = [timing_mx_log timing_mx];
+    av_level_log = [av_level_log av_level];
+  end
+
+  figure(2); clf;
+  mx = max(abs(rx)); 
+  subplot(211); plot(rx); axis([0 Nsam -mx mx]);
+  subplot(212); hold on;
+  plot(w_log,timing_mx_log,'b+-;timing mx;');
+  plot(w_log,av_level_log,'g+-;av level;');
+  hold off;
 endfunction
 
 
@@ -988,10 +1323,16 @@ endfunction
 format;
 more off;
 
-init_cml('/home/david/Desktop/cml/');
+init_cml('~/cml/');
 
-run_single(6) 
+%run_single(10);
 %run_curves
 %run_curves_estimators
-%acquisition_histograms
-%acquisition_test
+%acquisition_histograms(fin_en=0, foff_hz=-15, EbNoAWGN=-1, EbNoHF=3)
+%acquisition_test(Ntests=10, EbNodB=-1, foff_hz=0, hf_en=0, verbose=1);
+%sync_metrics('freq')
+%run_curves_snr
+%acquisition_dev(Ntests=10, EbNodB=100, foff_hz=0)
+%acquistion_curves
+%debug_false_sync
+metric_fbf("~/Desktop/false_sync.wav")
